
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.LinkedList;

public class Graph1 {

  public ArrayList<Node> nodeArray = new ArrayList<Node>();
  public ArrayList<Node> selectedNodes = new ArrayList<Node>();
  public ArrayList<NodeSet> nodeSets = new ArrayList<NodeSet>();
  private boolean isForceDirectedLayoutActive = true;
  int SPRING_REST_LENGTH = 50;

  public Graph1() {
  }

  public int getNumNodes() {
    return nodeArray.size();
  }

  public Node getNode(int index) {
    if (index < 0 || index >= nodeArray.size()) {
      return null;
    }
    return nodeArray.get(index);
  }

  public ArrayList<Node> getNodes() {
    return nodeArray;
  }

  public void addNode(Node n) {
    if (!nodeArray.contains(n)) {
      nodeArray.add(n);
    }
  }

  public void addEdge(Node n1, Node n2) {
    if (n1 == n2 || n1 == null || n2 == null) {
      return;
    }
    if (!n1.neighbours.contains(n2)) {
      n1.neighbours.add(n2);
    }
    if (!n2.neighbours.contains(n1)) {
      n2.neighbours.add(n1);
    }
  }

  public void addEdge(int nodeIndex1, int nodeIndex2) {
    addEdge(getNode(nodeIndex1), getNode(nodeIndex2));
  }

  public boolean areNodesAdjacent(Node n1, Node n2) {
    if (n1.neighbours.size() > n2.neighbours.size()) {
      return n2.neighbours.contains(n1);
    }
    return n1.neighbours.contains(n2);
  }

  // Note that this won't remove the node from the set of selected nodes;
  // that must be done by the caller, if so desired.
  private void deleteNode(Node n) {
    for (int i = 0; i < n.neighbours.size(); ++i) {
      Node otherNode = n.neighbours.get(i);
      otherNode.neighbours.remove(n);
    }
    n.neighbours.clear();
    nodeArray.remove(n);
    //TODO ... remove from any NodeSets the node belongs to
  }

  public void removeEdge(Node n1, Node n2) {
    if (n1 == n2 || n1 == null || n2 == null) {
      return;
    }
    if (n1.neighbours.contains(n2)) {
      n1.neighbours.remove(n2);
    }
    if (n2.neighbours.contains(n1)) {
      n2.neighbours.remove(n1);
    }
  }

  public int getNumSelectedNodes() {
    return selectedNodes.size();
  }

  public boolean isNodeSelected(Node node) {
    return selectedNodes.contains(node);
  }

  public ArrayList<Node> getSelectedNodes() {
    return selectedNodes;
  }

  public void selectNode(Node node) {
    if (!isNodeSelected(node)) {
      selectedNodes.add(node);
    }
  }

  public void deselectNode(Node node) {
    selectedNodes.remove(node);
  }

  public void selectByLabel(String s) {
    for (int j = 0; j < nodeArray.size(); ++j) {
      Node n = nodeArray.get(j);
      if (n.label.equals(s)) {
        selectNode(n);
      }
    }
  }

  public void shrinkSelection() {
    if (getNumSelectedNodes() > 0) {
      ArrayList<Node> nodes = getSelectedNodes();
      for (int j = 0; j < nodes.size(); ++j) {
        Node n = nodes.get(j);
        boolean hasNeighboursThatAreNotSelected = false;
        ArrayList<Node> adjacentNodes = n.neighbours;
        for (int i = 0; i < adjacentNodes.size(); ++i) {
          Node n2 = adjacentNodes.get(i);
          if (!isNodeSelected(n2)) {
            hasNeighboursThatAreNotSelected = true;
            break;
          }
        }
        n.TMP_color = hasNeighboursThatAreNotSelected
                ? Node.TMP_BLACK : Node.TMP_WHITE;
      }
      for (int i = getSelectedNodes().size() - 1; i >= 0; --i) {
        Node n = getSelectedNodes().get(i);
        if (n.TMP_color == Node.TMP_BLACK) {
          getSelectedNodes().remove(i);
        }
      }
    }
  }

  public void growSelection(int radius) {
    ArrayList<Node> setOfNodesToSelect = new ArrayList<Node>();
    findNeighbourhoodOfNodes(
            selectedNodes, radius, setOfNodesToSelect);
    for (int i = 0; i < setOfNodesToSelect.size(); ++i) {
      selectNode(setOfNodesToSelect.get(i));
    }
  }
  // Let C be the set of currently selected nodes,
  // and let S be the set of nodes 1 edge away from nodes in C and not already in C.
  // Growing the selection by "one step" would simply add S to C (i.e. C <- C U S).
  // To grow by "less than one step", this method selects the
  // one (or many) nodes in S that are adjacent to the greatest number
  // of nodes in C.  We assume that node (or nodes)
  // will be more interesting to the user.

  public void growSelectionByLessThanOneStep() {
    ArrayList<ArrayList<Node>> setsOfNodes = new ArrayList<ArrayList<Node>>();
    findNeighbourhoodOfNodesPartitionedByDistance(selectedNodes, 1, setsOfNodes);
    ArrayList<Node> setOfNearbyNodes = setsOfNodes.get(1);
    NodeAndScore[] nodeAndScore = new NodeAndScore[setOfNearbyNodes.size()];
    for (int i = 0; i < setOfNearbyNodes.size(); ++i) {
      Node n = setOfNearbyNodes.get(i);
      int score = 0;
      for (int j = 0; j < n.neighbours.size(); ++j) {
        Node neighbour = n.neighbours.get(j);
        if (isNodeSelected(neighbour)) {
          ++score;
        }
      }
      nodeAndScore[i] = new NodeAndScore(n, score);
    }
    Arrays.sort(nodeAndScore, new NodeAndScoreComparator());

    // Select the one (or many) nodes with the highest score
    for (int i = nodeAndScore.length - 1; i >= 0; --i) {
      if (nodeAndScore[i].score == nodeAndScore[nodeAndScore.length - 1].score) {
        selectNode(nodeAndScore[i].node);
      } else {
        break;
      }
    }
  }

  public void selectAllNodes() {
    selectedNodes.clear();
    for (int i = 0; i < nodeArray.size(); ++i) {
      selectedNodes.add(nodeArray.get(i));
    }
  }

  public void deselectAllNodes() {
    selectedNodes.clear();
  }

  public void invertSelectedNodes() {
    ArrayList<Node> newlySelectedNodes = new ArrayList<Node>();
    for (int i = 0; i < nodeArray.size(); ++i) {
      Node n = nodeArray.get(i);
      if (!selectedNodes.contains(n)) {
        newlySelectedNodes.add(n);
      }
    }
    selectedNodes = newlySelectedNodes;
  }

  public void deleteSelectedNodes() {
    for (int i = 0; i < selectedNodes.size(); ++i) {
      deleteNode(selectedNodes.get(i));
    }
    selectedNodes.clear();
  }

  // The return value of this can be used to index into nodeArray.
  public int getIndexOfNode(Node n) {
    assert n != null;

    // If the cached index is NOT correct ...
    int index = n.getIndex();
    if (index < 0 || index >= nodeArray.size() || getNode(index) != n) {
      // ... recompute all the indices
      // (this takes linear time, but subsequent calls
      // for this and other nodes will be fast,
      // until the next time the cached indices are
      // made invalid because some nodes have been
      // added or deleted)
      for (int i = 0; i < nodeArray.size(); ++i) {
        nodeArray.get(i).setIndex(i);
      }
    }

    // assert that the cached index IS now correct
    assert getNode(n.getIndex()) == n;

    // return the cached index
    return n.getIndex();
  }

  public Node findNearestNode(float x, float y, float minDistanceThreshold) {
    Node nearestNode = null;
    float smallestDistanceSquared = 0;
    for (int i = 0; i < nodeArray.size(); ++i) {
      Node n = nodeArray.get(i);
      float dx = x - n.x;
      float dy = y - n.y;
      float distanceSquared = dx * dx + dy * dy;
      if (nearestNode == null || distanceSquared < smallestDistanceSquared) {
        smallestDistanceSquared = distanceSquared;
        nearestNode = n;
      }
    }
    if (smallestDistanceSquared <= minDistanceThreshold * minDistanceThreshold) {
      return nearestNode;
    }
    return null;
  }

  public void setShapeTypeOfNodes(ArrayList<Node> nodes, int nodeShape) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      n.nodeShape = nodeShape;
    }
  }

  public void setShowLabelsOfNodes(ArrayList<Node> nodes, boolean flag) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      n.showLabel = flag;
    }
  }

  public void toggleShowLabelsOfNodes(ArrayList<Node> nodes) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      n.showLabel = !n.showLabel;
    }
  }

  public void setFixedFlagOfNodes(ArrayList<Node> nodes, boolean flag) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      n.isFixed = flag;
    }
  }

  public void randomizePositionsOfNodes(
          // If null, the randomization is applied to all nodes in the network,
          // otherwise it is only applied to the given ``nodes''.
          ArrayList<Node> nodes,
          // If true, even fixed nodes are randomized.
          boolean randomizeFixedNodes,
          // If true, the random displacements are proportional
          // to the extents of the current layout of the nodes.
          // If false, the randomization is chosen to uniformly
          // fill the given ``boundingRectangle''.
          boolean localized,
          // If non-null, the randomized positions will not fall
          // outside this rectangle.
          Rect2D boundingRectangle) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    Rect2D rectangle = new Rect2D();
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      if (randomizeFixedNodes || !n.isFixed) {
        rectangle.bound(new Vector2D(n.x, n.y));
      }
    }
    float distance = rectangle.getDiagonal().length();
    if (distance < SPRING_REST_LENGTH / 2) {
      distance = SPRING_REST_LENGTH / 2;
    }
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      if (randomizeFixedNodes || !n.isFixed) {
        if (localized || boundingRectangle == null) {
          n.x += distance * (Math.random() - 0.5f);
          n.y += distance * (Math.random() - 0.5f);
        } else {
          n.x = (float) (boundingRectangle.getMin().x() + Math.random() * boundingRectangle.getDiagonal().x());
          n.y = (float) (boundingRectangle.getMin().y() + Math.random() * boundingRectangle.getDiagonal().y());
        }

        if (boundingRectangle != null) {
          // bound the node's position

          if (n.x < boundingRectangle.getMin().x()) {
            n.x = boundingRectangle.getMin().x();
          } else if (n.x > boundingRectangle.getMax().x()) {
            n.x = boundingRectangle.getMax().x();
          }

          if (n.y < boundingRectangle.getMin().y()) {
            n.y = boundingRectangle.getMin().y();
          } else if (n.y > boundingRectangle.getMax().y()) {
            n.y = boundingRectangle.getMax().y();
          }
        }
      }
    }
  }

  public float computeClusteringCoefficientOfNode(Node n) {
    int numAdjacentNeighbourPairs = 0;
    for (int i = 0; i < n.neighbours.size() - 1; ++i) {
      Node neighbour1 = n.neighbours.get(i);
      for (int j = i + 1; j < n.neighbours.size(); ++j) {
        Node neighbour2 = n.neighbours.get(j);
        if (areNodesAdjacent(neighbour1, neighbour2)) {
          ++numAdjacentNeighbourPairs;
        }
      }
    }
    int totalPossible = n.neighbours.size() * (n.neighbours.size() - 1) / 2;
    return numAdjacentNeighbourPairs / (float) totalPossible;
  }

  //->untested
  public void computeHubs() {
    if (getNumNodes() > 0) {
      Node[] nodeArray = getArrayOfNodesInAscendingOrderOfDegree();
      Node firstNewlySelectedNode = null;
      // Search for highest-degree node that is not yet selected
      int i;
      for (i = nodeArray.length - 1; i >= 0; --i) {
        Node n = nodeArray[i];
        if (!isNodeSelected(n)) {
          firstNewlySelectedNode = n;
          selectNode(n);
          break;
        }
      }
      // Now select all other nodes of the same degree
      for (--i; i >= 0; --i) {
        Node n = nodeArray[i];
        if (n.neighbours.size() == firstNewlySelectedNode.neighbours.size()) {
          selectNode(n);
        } else {
          break;
        }
      }
    }
  }

  public Vector2D computeGeometricCentreOfNodes(
          // If null, the centre of all nodes in the network is returned,
          // otherwise only the given ``nodes'' are considered.
          ArrayList<Node> nodes) {
    return getBoundingRectangle(nodes).getCentre();
  }

  /**
   * Returns the bounding rectangle of the nodes in 2D. If the given vector
   * of nodes is null, calculates and returns the bounding rectangle of the
   * entire network.
   */
  public Rect2D getBoundingRectangle(ArrayList<Node> nodes) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    Rect2D rectangle = new Rect2D();
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      rectangle.bound(new Vector2D(n.x, n.y));
    }
    return rectangle;
  }

  /**
   * Computers the centroid of the given nodes in 2 dimensions, or the entire network if
   * the given vector is null.
   */
  public Vector2D computeCentroidOfNodes(
          // If null, the centroid of all nodes in the network is returned,
          // otherwise only the given ``nodes'' are considered.
          ArrayList<Node> nodes) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    Vector2D centroid = new Vector2D(0, 0);
    int numNodes = nodes.size();
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      centroid.v[0] += n.x;
      centroid.v[1] += n.y;
    }
    centroid = Vector2D.mult(centroid, 1.0f / numNodes);
    return centroid;
  }

  /**
   * Centres the nodes in two dimenstion about the given centre. If the given set
   * of vectors is null, centres all the nodes in the network about this point.
   */
  public void centreNodes(ArrayList<Node> nodes, Vector2D newCentre) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    Vector2D currentCentre = computeGeometricCentreOfNodes(nodes);
    Vector2D shift = Vector2D.diff(newCentre, currentCentre);
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      n.x += shift.x();
      n.y += shift.y();
    }
  }

  // This is used to sort nodes by some "score",
  // which may be an angle or other metric associated
  // with each node.
  private class NodeAndScore {

    public Node node;
    public float score;

    public NodeAndScore(Node n, float s) {
      node = n;
      score = s;
    }
  }

  private class NodeAndScoreComparator implements Comparator<NodeAndScore> {

    public int compare(NodeAndScore a, NodeAndScore b) {
      return (a.score < b.score) ? -1 : ((a.score > b.score) ? 1 : 0);
    }
  }

  // returns true if successful
  public boolean projectAndEvenlyDistributeNodesOnBestFittingLine(
          // If null, all nodes in the network are projected,
          // otherwise only the given ``nodes'' are considered.
          ArrayList<Node> nodes) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    int numNodes = nodes.size();
    Vector2D[] points = new Vector2D[numNodes];
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      points[i] = new Vector2D(n.x, n.y);
    }
    Vector2D e1 = new Vector2D(), e2 = new Vector2D();
    double[] eigenvalues = new double[2];
    boolean success = Vector2D.principleComponentAnalysis(
            points, e1, e2, eigenvalues);
    if (success) {
      Vector2D centroid = computeCentroidOfNodes(nodes);
      NodeAndScore[] nodeAndDotProduct = new NodeAndScore[numNodes];
      for (int i = 0; i < nodes.size(); ++i) {
        Node n = nodes.get(i);
        float dotProduct = Vector2D.dot(Vector2D.diff(new Vector2D(n.x, n.y), centroid), e1);
        nodeAndDotProduct[i] = new NodeAndScore(n, dotProduct);
      }

      // sort the nodes by their dot product with ``e1''
      Arrays.sort(nodeAndDotProduct, new NodeAndScoreComparator());

      Vector2D endpoint1 = Vector2D.mult(e1, nodeAndDotProduct[0].score);
      Vector2D endpoint2 = Vector2D.mult(e1, nodeAndDotProduct[nodeAndDotProduct.length - 1].score);
      float lengthOfLineSegment = Vector2D.diff(endpoint2, endpoint1).length();
      float spacing = lengthOfLineSegment / (nodeAndDotProduct.length - 1);

      for (int i = 0; i < nodeAndDotProduct.length; ++i) {
        Vector2D location = Vector2D.sum(endpoint1, Vector2D.mult(e1, i * spacing));
        nodeAndDotProduct[i].node.x = location.x() + centroid.x();
        nodeAndDotProduct[i].node.y = location.y() + centroid.y();
      }
    }
    return success;
  }

  public void projectAndEvenlyDistributeNodesOnBestFittingCircle(
          // If null, all nodes in the network are projected,
          // otherwise only the given ``nodes'' are considered.
          ArrayList<Node> nodes) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    Vector2D centroid = computeCentroidOfNodes(nodes);
    int numNodes = nodes.size();
    NodeAndScore[] nodeAndAngle = new NodeAndScore[numNodes];
    float averageRadius = 0;
    for (int i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      float relativeX = n.x - centroid.x();
      float relativeY = n.y - centroid.y();
      float radius = (float) Math.sqrt(relativeX * relativeX + relativeY * relativeY);
      float theta = computeAngle(relativeX, relativeY);

      averageRadius += radius;
      nodeAndAngle[i] = new NodeAndScore(n, theta);
    }
    averageRadius /= numNodes;

    // sort the nodes by their angle
    Arrays.sort(nodeAndAngle, new NodeAndScoreComparator());

    float delta_theta = 2 * (float) Math.PI / nodeAndAngle.length;
    // We want to give the nodes a uniform spacing around
    // a circle, such that node i has angle
    //    offset + i*delta_theta
    // and we would like to choose offset such that the
    // current angles of nodes are changed as little
    // as possible.
    // So, for each node, we compute the offset required
    // to maintain its current angle, and we find the
    // average of all such offsets.
    float average_offset = 0;
    for (int i = 0; i < nodeAndAngle.length; ++i) {
      average_offset += nodeAndAngle[i].score - i * delta_theta;
    }
    average_offset /= nodeAndAngle.length;
    // Now give the nodes even spacing and equal radius.
    for (int i = 0; i < nodeAndAngle.length; ++i) {
      Node n = nodeAndAngle[i].node;
      float theta = average_offset + i * delta_theta;
      n.x = averageRadius * (float) Math.cos(theta) + centroid.x();
      n.y = averageRadius * (float) Math.sin(theta) + centroid.y();
    }
  }

  // If the 3 given points lie on a circle,
  // this routine finds the center of that circle and returns true.
  // If the 3 points lie on a straight line, this routine returns false.
  public static boolean doPointsLieOnCircle(Vector2D p1, Vector2D p2, Vector2D p3, Vector2D centerOfCircle) {
    Vector2D v1 = Vector2D.diff(p2, p1);
    Vector2D midpoint1 = Vector2D.sum(p1, Vector2D.mult(v1, 0.5f));
    Vector2D direction1 = new Vector2D(v1.y(), -v1.x());
    Ray2D ray1 = new Ray2D(midpoint1, direction1);

    Vector2D v2 = Vector2D.diff(p3, p2);
    Vector2D midpoint2 = Vector2D.sum(p2, Vector2D.mult(v2, 0.5f));
    Vector2D direction2 = new Vector2D(v2.y(), -v2.x());
    Ray2D ray2 = new Ray2D(midpoint2, direction2);

    return Ray2D.doRaysIntersect(ray1, ray2, centerOfCircle);
  }

  public void projectAndEvenlyDistributeNodesOnBestFittingArc(
          // If null, all nodes in the network are projected,
          // otherwise only the given ``nodes'' are considered.
          ArrayList<Node> nodes,
          // The routine computes a best-fitting arc for the nodes,
          // and prior to projecting the nodes onto the arc,
          // the routine increases (or decreases) the angle of the arc
          // by the angle given here.  This is useful to allow the user to
          // interactively adjust the angle during dragging.
          // Pass in zero for this if no change to the best-fitting arc is desired.
          float increaseInArcAngle // in radians; can be negative, in which case a decrease occurs
          ) {
    if (nodes == null) {
      nodes = nodeArray;
    }
    if (nodes.size() < 3) {
      return;
    }
    Vector2D centroid = computeCentroidOfNodes(nodes);

    // Find max distance from node to centroid.
    // This will be used later to compute a scale factor.
    float maxDistanceFromCentroid = 0;
    int i;
    for (i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      Vector2D p = new Vector2D(n.x, n.y);
      float distance = Vector2D.diff(centroid, p).length();
      if (distance > maxDistanceFromCentroid) {
        maxDistanceFromCentroid = distance;
      }
    }

    Vector2D averageCenter = new Vector2D();
    int numCircles = 0;
    for (i = 0; i < nodes.size() - 2; ++i) {
      Node n1 = nodes.get(i);
      Vector2D p1 = new Vector2D(n1.x, n1.y);
      Node n2 = nodes.get(i + 1);
      Vector2D p2 = new Vector2D(n2.x, n2.y);
      Node n3 = nodes.get(i + 2);
      Vector2D p3 = new Vector2D(n3.x, n3.y);
      Vector2D centerOfCircle = new Vector2D();
      if (doPointsLieOnCircle(p1, p2, p3, centerOfCircle)) {
        ++numCircles;
        averageCenter.v[0] += centerOfCircle.x();
        averageCenter.v[1] += centerOfCircle.y();
      }
    }
    if (numCircles > 0) {
      averageCenter.v[0] /= numCircles;
      averageCenter.v[1] /= numCircles;
    } else {
      averageCenter.copy(centroid);
    }

    int numNodes = nodes.size();
    NodeAndScore[] nodeAndAngle = new NodeAndScore[nodes.size()];
    float averageRadius = 0;
    for (i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      float relativeX = n.x - averageCenter.x();
      float relativeY = n.y - averageCenter.y();
      float radius = (float) Math.sqrt(relativeX * relativeX + relativeY * relativeY);
      float theta = computeAngle(relativeX, relativeY);
      // Ensures theta is in [0,2*pi]
      if (theta < 0) {
        theta += 2 * (float) Math.PI;
      }

      averageRadius += radius;
      nodeAndAngle[i] = new NodeAndScore(n, theta);
    }
    averageRadius /= nodes.size();

    // sort the nodes by their angle
    Arrays.sort(nodeAndAngle, new NodeAndScoreComparator());

    // find the largest angular gap between consecutive points
    float largestAngularGap = 0;
    int indexOfLargestAngularGap = 0;
    float angularGap;
    for (i = 0; i < nodes.size() - 1; ++i) {
      angularGap = nodeAndAngle[i + 1].score - nodeAndAngle[i].score;
      if (angularGap > largestAngularGap) {
        largestAngularGap = angularGap;
        indexOfLargestAngularGap = i;
      }
    }
    // also check the gap between the first and last points
    angularGap = (nodeAndAngle[0].score + 2 * (float) Math.PI) - nodeAndAngle[nodes.size() - 1].score;
    if (angularGap > largestAngularGap) {
      largestAngularGap = angularGap;
      indexOfLargestAngularGap = nodes.size() - 1;
    }

    float theta0 = nodeAndAngle[(indexOfLargestAngularGap + 1) % nodes.size()].score;
    float arcAngle = 2 * (float) Math.PI - largestAngularGap;

    if (increaseInArcAngle != 0) { // modify the arc
      float maxArcAngle = 0.95f * 2 * (float) Math.PI * (nodes.size() - 1) / nodes.size();
      float minArcAngle = 15 / 360.0f * 2 * (float) Math.PI;

      float newArcAngle = arcAngle + increaseInArcAngle;
      if (newArcAngle > maxArcAngle) {
        newArcAngle = maxArcAngle;
      } else if (newArcAngle < minArcAngle) {
        newArcAngle = minArcAngle;
      }

      increaseInArcAngle = newArcAngle - arcAngle;
      theta0 -= increaseInArcAngle / 2;
      arcAngle = newArcAngle;
    }

    float deltaTheta = arcAngle / (nodes.size() - 1);
    for (i = 0; i < nodes.size(); ++i) {
      Node n = nodeAndAngle[i].node;
      float theta = theta0 + ((i - (indexOfLargestAngularGap + 1) + nodes.size()) % nodes.size()) * deltaTheta;
      n.x = averageRadius * (float) Math.cos(theta) + averageCenter.x();
      n.y = averageRadius * (float) Math.sin(theta) + averageCenter.y();
    }

    Vector2D newCentroid = computeCentroidOfNodes(nodes);

    // Find max distance from node to centroid.
    // This will be used later to compute a scale factor.
    float newMaxDistanceFromNewCentroid = 0;
    for (i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      Vector2D p = new Vector2D(n.x, n.y);
      float distance = Vector2D.diff(newCentroid, p).length();
      if (distance > newMaxDistanceFromNewCentroid) {
        newMaxDistanceFromNewCentroid = distance;
      }
    }

    // scale and translate the nodes to restore their original centroid and "size"
    // (as measured by maxDistanceFromCentroid)
    float scaleFactor = (maxDistanceFromCentroid > 0 && newMaxDistanceFromNewCentroid > 0) ? (maxDistanceFromCentroid / newMaxDistanceFromNewCentroid) : 1;
    for (i = 0; i < nodes.size(); ++i) {
      Node n = nodes.get(i);
      Vector2D relativePosition = Vector2D.diff(new Vector2D(n.x, n.y), newCentroid);
      Vector2D newPosition = Vector2D.sum(centroid, Vector2D.mult(relativePosition, scaleFactor));
      n.x = newPosition.x();
      n.y = newPosition.y();
    }

  }

  public abstract class NodeVisitor {
    // Called by a routine traversing the graph.
    // Returns false if the visitor wants the traversal to be stopped,
    // which may happen, for example, if the visitor finds the one
    // node it was searching for.

    public abstract boolean visit(
            Node node,
            Node predecessorNode,
            int distanceToStartNode, // in edges
            Graph1 network);
  }

  public void breadthFirstTraversal(
          ArrayList<Node> rootNodes, // a vector of starting nodes for the traversal
          NodeVisitor visitor) {
    if (rootNodes.isEmpty()) {
      return;
    }

    // During the traversal, nodes are either "white" (not yet visited),
    // "grey" (added to list of nodes to visit next),
    // or "black" (already visited).

    // Initialization.
    for (int i = 0; i < nodeArray.size(); ++i) {
      Node n = nodeArray.get(i);
      n.TMP_color = Node.TMP_WHITE;
      n.TMP_predecessorNode = null;
      n.TMP_distanceFromStartNode = -1;
    }

    LinkedList<Node> greyNodes = new LinkedList<Node>();
    for (int i = 0; i < rootNodes.size(); ++i) {
      Node rootNode = rootNodes.get(i);
      rootNode.TMP_color = Node.TMP_GREY;
      rootNode.TMP_distanceFromStartNode = 0;
      greyNodes.addFirst(rootNode);
    }
    do {
      // Get the next node to visit
      Node n = greyNodes.removeLast();

      // Mark all adjacent nodes as grey.
      ArrayList<Node> adjacentNodes = n.neighbours;
      for (int i = 0; i < adjacentNodes.size(); ++i) {
        Node n2 = adjacentNodes.get(i);
        if (n2.TMP_color == Node.TMP_WHITE) {
          n2.TMP_color = Node.TMP_GREY;
          n2.TMP_predecessorNode = n;
          n2.TMP_distanceFromStartNode = n.TMP_distanceFromStartNode + 1;
          greyNodes.addFirst(n2);
        }
      }

      if (visitor != null) {
        // Visit the node
        if (!visitor.visit(n, n.TMP_predecessorNode, n.TMP_distanceFromStartNode, this)) {
          break;
        }
      }

      // Mark the node as having been visited
      n.TMP_color = Node.TMP_BLACK;

    } while (!greyNodes.isEmpty());
  }

  public void breadthFirstTraversal(
          Node rootNode, // a single starting node for the traversal
          NodeVisitor visitor) {
    if (rootNode == null) {
      return;
    }
    ArrayList<Node> vectorOfRootNodes = new ArrayList<Node>();
    vectorOfRootNodes.add(rootNode);
    breadthFirstTraversal(vectorOfRootNodes, visitor);
  }

  public void findNeighbourhoodOfNodesPartitionedByDistance(
          ArrayList<Node> rootNodes, // a vector of nodes that form the "centre" of the neighbourhood
          int radius, // the radius of the neighbourhood to find, in edges from the ``rootNodes''; -1 means infinity

          // This is where the neighbourhood is returned,
          // with nodes organized classified by distance from the ``rootNodes''.
          // The caller should pass in an empty vector object ( new ArrayList<ArrayList<Node>>() ).
          // What is returned is a vector of vectors of nodes.
          // There's one "sub" vector for each distance from the root nodes.
          // Afterwards, the caller can index into the "top level" vector using distance.
          // So, the root nodes are in the first "sub" vector (at index 0 in the "top level" vector),
          // and all nodes that are 1 edge away from the root nodes are in the next "sub" vector,
          // and all nodes 2 edges away are in the next vector, etc.
          ArrayList<ArrayList<Node>> setsOfNodes) {
    if (rootNodes.isEmpty()) {
      return;
    }
    setsOfNodes.clear();

    class MyNodeVisitor extends NodeVisitor {

      private int radius;
      ArrayList<ArrayList<Node>> setsOfNodes;

      public MyNodeVisitor(int r, ArrayList<ArrayList<Node>> s) {
        radius = r;
        setsOfNodes = s;
      }

      public boolean visit(
              Node node,
              Node predecessorNode,
              int distanceToStartNode,
              Graph1 network) {
        if ( /*radius==-1 means infinity, i.e. don't stop*/radius >= 0 && distanceToStartNode > radius) // stop the traversal
        {
          return false;
        }

        // Add the node to ``setsOfNodes''
        while (node.TMP_distanceFromStartNode >= setsOfNodes.size()) {
          setsOfNodes.add(new ArrayList<Node>());
        }
        (setsOfNodes.get(node.TMP_distanceFromStartNode)).add(node);

        return true;
      }
    }
    ;
    NodeVisitor visitor = new MyNodeVisitor(radius, setsOfNodes);

    breadthFirstTraversal(rootNodes, visitor);
  }

  public void findNeighbourhoodOfNodePartitionedByDistance(
          Node rootNode, // a single centre of the neighbourhood
          int radius,
          ArrayList<ArrayList<Node>> setsOfNodes) {
    if (rootNode == null) {
      return;
    }
    ArrayList<Node> arrayOfRootNodes = new ArrayList<Node>();
    arrayOfRootNodes.add(rootNode);
    findNeighbourhoodOfNodesPartitionedByDistance(arrayOfRootNodes, radius, setsOfNodes);
  }

  // Similar to findNeighbourhoodOfNodesPartitionedByDistance(),
  // except the set of nodes returned is flat instead of being
  // partitioned by distance.
  public void findNeighbourhoodOfNodes(
          ArrayList<Node> rootNodes, // a vector of nodes that form the "centre" of the neighbourhood
          int radius, // the radius of the neighbourhood to find, in edges from the ``rootNodes''; -1 means infinity
          ArrayList<Node> setOfNodes) {
    setOfNodes.clear();
    ArrayList<ArrayList<Node>> setsOfNodes = new ArrayList<ArrayList<Node>>();
    findNeighbourhoodOfNodesPartitionedByDistance(rootNodes, radius, setsOfNodes);

    // "flatten" the sets into a single set
    for (int i = 0; i < setsOfNodes.size(); ++i) {
      ArrayList<Node> subArray = setsOfNodes.get(i);
      for (int j = 0; j < subArray.size(); ++j) {
        Node n = subArray.get(j);
        setOfNodes.add(n);
      }
    }
  }

  public void findNeighbourhoodOfNode(
          Node rootNode, // a single centre of the neighbourhood
          int radius,
          ArrayList<Node> setOfNodes) {
    ArrayList<Node> arrayOfRootNodes = new ArrayList<Node>();
    arrayOfRootNodes.add(rootNode);
    findNeighbourhoodOfNodes(arrayOfRootNodes, radius, setOfNodes);
  }

  private static float computeAngle(float x, float y) {
    float r = (float) Math.sqrt(x * x + y * y);
    if (r <= 0) {
      return 0;
    }
    float angle = (float) Math.asin(y / r);
    if (x < 0) {
      angle = (float) Math.PI - angle;
    }
    // Note that angle is in [-pi/2,3*pi/2]
    return angle;
  }

  private static float computeAngle(Vector2D v) {
    return computeAngle(v.x(), v.y());
  }

  // Lays out the nodes that are within the given radius of the given central nodes
  // in concentric circles.
  public void performConcentricCircleLayout(ArrayList<Node> centralNodes, int neighbourhoodRadius, boolean fixNodes) {
    if (centralNodes.size() < 1) {
      return;
    }
    boolean multipleCentralNodes = centralNodes.size() > 1;
    ArrayList<ArrayList<Node>> setsOfNodes = new ArrayList<ArrayList<Node>>();
    findNeighbourhoodOfNodesPartitionedByDistance(centralNodes, neighbourhoodRadius, setsOfNodes);
    Vector2D centre = computeGeometricCentreOfNodes(centralNodes);

    int i;

    // Save the original positions of the nodes
    // that will be placed on the inner-most circle.
    // These will be used later.
    int indexOfInnerMostCircle = multipleCentralNodes ? 0 : 1;
    ArrayList<Node> nodesOnInnerMostCircle = null;
    Vector2D originalPositionsOfNodesOnInnerMostCircle[] = null;
//System.out.println("size: "+setsOfNodes.size() + " index: "  +  indexOfInnerMostCircle);
    if (setsOfNodes.size() > indexOfInnerMostCircle) {
      nodesOnInnerMostCircle = setsOfNodes.get(indexOfInnerMostCircle);
      originalPositionsOfNodesOnInnerMostCircle = new Vector2D[nodesOnInnerMostCircle.size()];
      for (i = 0; i < nodesOnInnerMostCircle.size(); ++i) {
        Node n = nodesOnInnerMostCircle.get(i);
        originalPositionsOfNodesOnInnerMostCircle[i] = new Vector2D(n.x, n.y);
      }
    }

    // Initalization: most nodes marked as unvisited (i.e. white).
    for (i = 0; i < setsOfNodes.size(); ++i) {
      ArrayList<Node> subArray = setsOfNodes.get(i);
      if (i == 0 && !multipleCentralNodes) {
        // Special case:
        // position, and mark as visited,
        // the one central node.
        Node n = centralNodes.get(0);
        n.TMP_color = Node.TMP_BLACK;
        n.x = centre.x();
        n.y = centre.y();
      } else if (i == setsOfNodes.size() - 1) {
        // Special case:
        // position, and mark as visited,
        // the outer-most nodes.
        float r = (multipleCentralNodes ? i + 1 : i) * SPRING_REST_LENGTH;
        for (int j = 0; j < subArray.size(); ++j) {
          Node n = subArray.get(j);
          n.TMP_color = Node.TMP_BLACK;
          float theta = j / (float) subArray.size() * 2 * (float) Math.PI;
          n.x = r * (float) Math.cos(theta) + centre.x();
          n.y = r * (float) Math.sin(theta) + centre.y();
        }
      } else {
        for (int j = 0; j < subArray.size(); ++j) {
          Node n = subArray.get(j);
          n.TMP_color = Node.TMP_WHITE;
        }
      }

      // Also mark all the nodes as fixed,
      // if the caller wants us to.
      if (fixNodes) {
        for (int j = 0; j < subArray.size(); ++j) {
          Node n = subArray.get(j);
          n.isFixed = true;
        }
      }
    }

    // Position the remaining white nodes.
    for (i = setsOfNodes.size() - 1; i >= (multipleCentralNodes ? 0 : 1); --i) {
      float r = (multipleCentralNodes ? i + 1 : i) * SPRING_REST_LENGTH;
      ArrayList<Node> subArray = setsOfNodes.get(i);
      int N = subArray.size();

      // Recall that the black nodes are already positioned.
      // We need to find white nodes and evenly space them between surrounding black nodes.
      // Specifically, we need to find pairs of indices (j1,j2) where node_j1 is black,
      // and is followed by un unbroken series of white nodes, which is followed by black node_j2.
      // For each such pair (j1,j2), we will evenly space the white nodes
      // between node_j1 and node_j2.
      int j1 = 0, j2 = 0;
      boolean weAreFinished = false;

      // Initialization: find the first positioned (black) node
      for (j1 = 0; j1 < N; ++j1) {
        if ((subArray.get(j1)).TMP_color == Node.TMP_BLACK) {
          break;
        }
      }
      j2 = j1;
      // Now j1 and j2 are both at a black node.

      while (!weAreFinished) {
        // Advance j1 to the next positioned (black) node that is followed by a white node
        while ((subArray.get((j1 + 1) % N)).TMP_color != Node.TMP_WHITE) {
          j1 = (j1 + 1) % N;
          if (j1 == j2) {
            // We've wrapped around to our starting point.
            weAreFinished = true;
            break;
          }
        }
        if (weAreFinished) {
          break;
        }

        // Advance j2 to the next positioned (black) node after j1
        j2 = (j1 + 1) % N;
        while ((subArray.get(j2)).TMP_color == Node.TMP_WHITE) {
          j2 = (j2 + 1) % N;
          if (j1 == j2) {
            // We've wrapped around to our starting point.
            weAreFinished = true;
            break;
          }
        }
        if (weAreFinished) {
          break;
        }

        // Evenly space all the white nodes between j1 and j2
        float theta_j1 = computeAngle(
                (subArray.get(j1)).x - centre.x(),
                (subArray.get(j1)).y - centre.y());
        float theta_j2 = computeAngle(
                (subArray.get(j2)).x - centre.x(),
                (subArray.get(j2)).y - centre.y());
        if (theta_j2 < theta_j1) {
          theta_j2 += 2 * Math.PI;
        }
        float deltaTheta = (theta_j2 - theta_j1) / ((j2 - j1 + N) % N);
        for (int j3 = (j1 + 1) % N; j3 != j2; j3 = (j3 + 1) % N) {
          float theta_j3 = deltaTheta * ((j3 - j1 + N) % N) + theta_j1;
          Node n = subArray.get(j3);
          n.TMP_color = Node.TMP_BLACK;
          n.x = r * (float) Math.cos(theta_j3) + centre.x();
          n.y = r * (float) Math.sin(theta_j3) + centre.y();
        }

        j1 = j2;
        // Now j1 and j2 are both at a black node.
      }

      // now position parent nodes in the next circle inwards
      j1 = j2 = 0;
      while (j2 < N) {
        while (j2 < N - 1
                && (subArray.get(j1)).TMP_predecessorNode
                == (subArray.get(j2 + 1)).TMP_predecessorNode) {
          ++j2;
        }


        // position the parent node of node_j1 and node_j2
        Node parentNode = (subArray.get(j1)).TMP_predecessorNode;
        if (parentNode != null && parentNode.TMP_color == Node.TMP_WHITE) {
          float theta_j1 = computeAngle(
                  (subArray.get(j1)).x - centre.x(),
                  (subArray.get(j1)).y - centre.y());
          float theta_j2 = computeAngle(
                  (subArray.get(j2)).x - centre.x(),
                  (subArray.get(j2)).y - centre.y());
          if (theta_j2 < theta_j1) {
            theta_j2 += 2 * Math.PI;
          }
          float theta_parent = (theta_j2 - theta_j1) / 2 + theta_j1;
          parentNode.TMP_color = Node.TMP_BLACK;
          parentNode.x = (r - SPRING_REST_LENGTH) * (float) Math.cos(theta_parent) + centre.x();
          parentNode.y = (r - SPRING_REST_LENGTH) * (float) Math.sin(theta_parent) + centre.y();
        }


        // advance to the next group of children
        j1 = j2 = j2 + 1;
      }

    } // for loop

    if (setsOfNodes.size() <= indexOfInnerMostCircle) {
      return;
    }
    // Finally, we rotate all the nodes by some angle
    // chosen to minimize how much the nodes on the inner-most
    // circle are moved from their original positions.
    //
    // To determine the angle of rotation to use,
    // we iterate over the nodes on the inner-most circle,
    // and find the average angle of rotation that
    // exists between their original position and
    // their currently assigned position.
    // We will then rotate *all* nodes by the negative
    // of that average angle, to move the nodes on the
    // inner-most circle back as close as possible to
    // their original positions.
    Vector2D averageRotationVector = new Vector2D();
    for (i = 0; i < nodesOnInnerMostCircle.size(); ++i) {
      Node n = nodesOnInnerMostCircle.get(i);

      Vector2D originalPosition = originalPositionsOfNodesOnInnerMostCircle[i];
      Vector2D originalRelativePosition = Vector2D.diff(originalPosition, centre);
      float originalAngle = computeAngle(originalRelativePosition);

      Vector2D currentPosition = new Vector2D(n.x, n.y);
      Vector2D currentRelativePosition = Vector2D.diff(currentPosition, centre);
      float currentAngle = computeAngle(currentRelativePosition);

      float rotationAngle = currentAngle - originalAngle;

      // To find the average rotation angle,
      // we might try doing something like
      //
      //    float averageRotationAngle = 0;
      //    for ( ... ) {
      //       ...
      //       float rotationAngle = ...
      //       averageRotationAngle += rotationAngle;
      //    }
      //    averageRotationAngle /= nodesOnInnerMostCircle.size();
      //
      // but this turns out to not work well.
      // For example, if there are two rotation angles equal to
      // 10 degrees and 350 degrees, the numerical averaging above
      // will compute their average to be 180 degrees,
      // whereas the average that we should be computing is
      // 0 degrees (or, equivalently, 360 degrees).
      // To compute the average correctly, we use a "rotation vector".
      //
      averageRotationVector = Vector2D.sum(
              averageRotationVector,
              new Vector2D((float) Math.cos(rotationAngle), (float) Math.sin(rotationAngle)));
    }
    float averageRotationAngle = computeAngle(averageRotationVector);
//System.out.println("based on " + nodesOnInnerMostCircle.size() + " nodes, the average angle is " + averageRotationAngle);
    float globalRotationAngle = -averageRotationAngle;
    float sineOfGlobalRotationAngle = (float) Math.sin(globalRotationAngle);
    float cosineOfGlobalRotationAngle = (float) Math.cos(globalRotationAngle);
    for (i = 0; i < setsOfNodes.size(); ++i) {
      ArrayList<Node> subArray = setsOfNodes.get(i);
      for (int j = 0; j < subArray.size(); ++j) {
        Node n = subArray.get(j);

        // perform the rotation
        float relativeX = n.x - centre.x();
        float relativeY = n.y - centre.y();
        n.x = cosineOfGlobalRotationAngle * relativeX - sineOfGlobalRotationAngle * relativeY + centre.x();
        n.y = sineOfGlobalRotationAngle * relativeX + cosineOfGlobalRotationAngle * relativeY + centre.y();
      }
    }

  }

  Node[] getArrayOfNodesInAscendingOrderOfDegree() {
    Node[] arrayOfNodes = nodeArray.toArray(new Node[getNumNodes()]);
    // Sort nodes in ascending order of degree
    Arrays.sort(
            arrayOfNodes,
            new Comparator() {

              public int compare(Object o1, Object o2) {
                Node a = (Node) o1;
                Node b = (Node) o2;
                int A = a.neighbours.size();
                int B = b.neighbours.size();
                return (A < B) ? -1 : ((A > B) ? 1 : 0);
              }
            });
    return arrayOfNodes;
  }

  public String getStatusString() {
    String status = "" + getNumNodes() + " nodes total; "
            + selectedNodes.size() + " nodes selected; ";
    return status;
  }

  private void clearAllNodeSets() {
    for (int i = 0; i < nodeSets.size(); ++i) {
      nodeSets.get(i).clear();
    }
    nodeSets.clear();
  }
}


