
import java.util.ArrayList;
import java.util.HashMap;

public class Node1{

    static HashMap<String, Object> attributesMax = new HashMap<String, Object>();
    public HashMap<String, Object> attributes = new HashMap<String, Object>();
    public ArrayList<Node> neighbours = new ArrayList<Node>();
    public float clusteringCoefficient = 0;
    public float coreLayerNumber = 0;
    // Each node may be part of 0 or 1 tree-like component,
    // and part of 0 or more "chain" components,
    // and part of 0 or more bicomponents (biconnected components).
    // The following data member stores references to these components.
    public ArrayList<NodeSet> nodeSets = new ArrayList<NodeSet>();
    public String label;
    public boolean showLabel = false;
    public String name;
    public float x = 0, y = 0; // position
    public float color_r = 0, color_g = 0, color_b = 0;
    public float alpha = 0.5f;
    // The NS_ prefix means "Node Shape"
    public static final int NS_CIRCLE = 0;
    public static final int NS_SQUARE = 1;
    public static final int NS_TRIANGLE = 2;
    public static final int NS_DIAMOND = 3;
    public static final int NUM_NODE_SHAPES = 4;
    public int nodeShape = NS_CIRCLE;
    public boolean isFixed = false;
    private boolean temporarilyHighlighted = false;
    /**
     * Used in the force-directed layout algorithms
     * to temporarily store components of pseudo "forces" acting on this node.
     * These components are later used to update the node's position.
     */
    public float forceX, forceY;
    /**
     * Other stuff used in the force-directed layout.
     */
    public float weight = 1; // non-negative
    public float numberOfOtherNodes; // for temporary storage only
    public float sumOfWeightsOfOtherNodes; // for temporary storage only
    /**
     * These data members are used for Breadth-First Traversal (BFT)
     * or Depth-First Traversal (DFT).
     * They are only of temporary use during a traversal,
     * afterwhich they can be ignored.
     * Before performing a traversal and using them,
     * they should be re-initialized to overwrite any garbage values
     * they may contain.
     */
    public static final int TMP_WHITE = 0;  // nodes not yet visited
    public static final int TMP_GREY = 1;   // nodes added to list of nodes to visit next
    public static final int TMP_BLACK = 2;  // nodes already visited
    public static final int TMP_IGNORE = 3; // do not traverse these nodes
    public int TMP_color = TMP_WHITE;
    public Node TMP_predecessorNode = null;
    public int TMP_distanceFromStartNode = 0;
    public int TMP_discoveryTime = 0;
    public int TMP_indexOfCurrentChild = -1;
    public int TMP_int1 = 0;
    // This is a temporary data member,
    // used during depth-first traversal to identify bicomponents
    //public int TMP_bicomponentSearch_back = 0;
    // This is another temporary data member,
    // used for neighbourhood comparison.
    // It is only of temporary use during neighbourhood comparison
    // afterwhich it can be ignored.
    // It does not need to be saved to, or loaded from, files.
    // Before using it,
    // it should be re-initialized to overwrite any garbage values
    // it may contain.
    //
    //public String TMP_neighbourhoodSignature;
    /**
     * This data member may be used by a client to "attach"
     * an arbitrary object to the Node, making the Node
     * more extensible.
     * However, since multiple clients may want to use this
     * data member to store their own data,
     * clients should adopt a policy of only using it
     * temporarily: for example, during a special computation,
     * if the client needs to temporarily store data associated
     * with each node, they could store it in an object referenced
     * by this data member, and then once they are finished
     * they could clean up by setting this data member back to null
     * so it's ready for the next client that will use it.
     */
    public Object clientData = null;
    // We assume that each node is "owned" by only one network.
    // Within a network, each node has a corresponding index
    // that can be used to look up the node in an array (or vector or something similar)
    // of nodes that is maintained by the network.
    // The same index also corresponds to the node's row and column
    // in the network's adjacency matrix.
    // To make it easier to determine the index for a given node,
    // we have the following data member that can be used to cache the node's index.
    // If a few nodes get deleted or added to the network,
    // causing the index stored here to become invalid,
    // we can simply lazily update it whenever we find it to be incorrect.
    // (The updating happens in Network.getIndexOfNode() )
    //
    private int index = -1;
    //private int zLayerIndex = 0;

    public Node1() {
    }

    public Node1(String s) {
        label = s;
    }

    public Node1(int _index) {
        index = _index;
    }

    // These two routines are useful if a client wants to loop over
    // only node sets of a given type t, in which case the client
    // could do something like
    //    Node n = ...
    //    for ( int i = 0; i < n.numNodeSetsOfGivenType(t); ++i ) {
    //       NodeSet ns = n.getNodeSetOfGivenType(t,i);
    //       ...
    //    }
    // or, if the client knows that a node is part of only one node set of a
    // given type, and wants to access it, the client could do
    //    assert n.numNodeSetsOfGivenType(t) == 1;
    //    NodeSet ns = n.getNodeSetOfGivenType(t,0);
    //    ...
    // or, if the client wants to test if a node is part of a node set
    // of a given type, the client can do
    //    if ( n.numNodeSetsOfGivenType(t) > 0 ) ...
    //
    /*public int numNodeSetsOfGivenType(int type) {
    int count = 0;
    for (int i = 0; i < nodeSets.size(); ++i) {
    if (nodeSets.get(i).type == type) {
    ++count;
    }
    }
    return count;
    }*/

    /*public NodeSet getNodeSetOfGivenType(int type, int index) {
    int count = 0;
    for (int i = 0; i < nodeSets.size(); ++i) {
    NodeSet ns = nodeSets.get(i);
    if (ns.type == type) {
    if (count == index) {
    return ns;
    }
    ++count;
    }
    }
    return null;
    }*/
    public boolean isTemporarilyHighlighted() {
        return temporarilyHighlighted;
    }

    public void setTemporarilyHighlighted(boolean flag) {
        temporarilyHighlighted = flag;
    }

    public void setIndex(int i) {
        index = i;
    }

    // Note: the index returned here is not guaranteed to be correct.
    // Clients who want an index guaranteed to be correct
    // should call Network.getIndexOfNode()
    public int getIndex() {
        return index;
    }

    /*public int getZLayerIndex() {
    return zLayerIndex;
    }

    public void setZLayerIndex(int zli) {
    zLayerIndex = zli;
    }*/

    /*int getCoreLayerNumber() {
    for (int i = 0; i < nodeSets.size(); ++i) {
    NodeSet ns = nodeSets.get(i);
    if (ns.type == NodeSet.NST_CORE_LAYER) {
    return ns.coreLayerNumber;
    }
    }
    return -1;
    }*/
    public static final int DIMENSION_DEGREE = 0;
    public static final int DIMENSION_CLUSTERING_COEFFICIENT = 1;
    public static final int DIMENSION_CORE_LAYER_NUMBER = 2;

    public int getNumDimensions() {
        return 3;
    }

    public String getAbbreviatedNameOfDimension(int i) {
        switch (i) {
            case DIMENSION_DEGREE:
                return "deg";
            case DIMENSION_CLUSTERING_COEFFICIENT:
                return "cc";
            case DIMENSION_CORE_LAYER_NUMBER:
                return "core";
        }
        assert false; // we should never reach this point
        return "???";
    }

    public float getValueAlongDimension(int i) {
        switch (i) {
            case DIMENSION_DEGREE:
                return neighbours.size();
            case DIMENSION_CLUSTERING_COEFFICIENT:
                return clusteringCoefficient;
            case DIMENSION_CORE_LAYER_NUMBER:
                return coreLayerNumber;
        }
        assert false; // we should never reach this point
        return -1;
    }

    public float getDegree() {
        return neighbours.size();
    }

    public float getClusteringCoefficient() {
        return clusteringCoefficient;
    }

} // End class

