
import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

public class FlockingDisplay {

  Graph g = new Graph();
  Flocking[] flocking;
  Comparator<Flocking> compareByX = new FlockingXComparator();
  Thread flockingThread;
  float rotX = 0;
  float rotY = 0;
  JoglFramework fw;
  Draw draw;
  ArrayList<Float> soundValue = new ArrayList<Float>();

  FlockingDisplay(JoglFramework _fw, Draw _draw) {
    fw = _fw;
    draw = _draw;
    for (int i = 0; i < 1000; i++) {
      g.addNode(new Node());
    }

    flocking = new Flocking[g.getNumNodes()];
    for (int i = 0; i < g.getNumNodes(); i++) {
      //flocking[i] = new Flocking((int) (Math.random() * fw.width), (int) (Math.random() * fw.height));
      flocking[i] = new Flocking(new Vector3D(
              (float) (Math.random() * fw.width),
              (float) (Math.random() * fw.height), 0));
    }

    Runnable r1 = new Runnable() {

      public void run() {
        try {
          while (true) {
            computeFlocking();
            Thread.sleep(10L);
          }
        } catch (InterruptedException iex) {
        }
      }
    };

    flockingThread = new Thread(r1);
    flockingThread.start();

  }

  public void draw() {
    ////auto rotate
    rotY += 0.5;
    rotX -= 0.5;
    rotX %= 360;
    rotY %= 360;

    ////draw spherical fluids
    draw.enableAlphaBlending(true);
    draw.beginTranslate(fw.width / 2, fw.height / 2, 0);
    draw.beginRotate(0f, 0f, 0f, rotX, rotY, 0);
    for (int i = 0; i < g.getNumNodes(); i++) {
      Node node = g.getNode(i);
      node.x = (float) flocking[i].getX();
      node.y = (float) flocking[i].getY();
      draw.setColor(0f, 0.5f, 1f, 0.4f);
      Vector3D sphericalVec = flocking[i].getCartesianFromSphericalCoords();
      draw.beginRotate(sphericalVec.x(), sphericalVec.y(), sphericalVec.z(),
              180 * flocking[i].sphericalPosPhi / (float) (Math.PI),
              //90,
              360 * flocking[i].sphericalPosTheta / (float) (Math.PI),
              //0,
              //(float) flocking[i].getThetaDeg());
              0);
      draw.drawEllipse(sphericalVec.x(), sphericalVec.y(), sphericalVec.z(), 20f, 3f, 60, true);
      draw.setColor(0.1f, 0.3f, flocking[i].sphericalPosPhi / (float) (Math.PI), 0.5f);
//      draw.drawLine(sphericalVec.x(), sphericalVec.y(), sphericalVec.z(),
//              sphericalVec.x(), sphericalVec.y(), sphericalVec.z() + 100);
      draw.drawLine(sphericalVec.x(), sphericalVec.y(), sphericalVec.z(),
              sphericalVec.x(), sphericalVec.y(), sphericalVec.z() + 50 + flocking[i].zoneVal * 200);
      draw.endRotate();
    }
    draw.endRotate();
    draw.endTranslate();
  }

  public void drawPlanisphere(float[] features) {

    int zoneX = 20;
    int zoneY = 200;
    int zoneW = 220;
    int zoneH = 190;
    int colNum = 5;
    int rowNum = 4;
    int subZoneNum = rowNum * colNum;
    float subZoneW = zoneW / colNum;
    float subZoneH = zoneH / rowNum;
    //int soundFeatureNum = 40;
    //int soundFeatureNumPerSubZone = soundFeatureNum / subZoneNum;
    //->hardcoded
    float soundFeatureDivider = 10;
    if (features != null) {
      soundValue.clear();
      for (int i = 0; i < subZoneNum; i++) {
        float featureVal = (float) Math.log1p(features[i] + features[i + 1]) / soundFeatureDivider;
        soundValue.add((Float) featureVal);
        draw.setColor(featureVal, 0f, 0f, 1f);
        //draw.drawRectdBottomAligned(x, y, w, (float) Math.log1p(features[i]) * 10);
        int rowIdx = i / colNum;
        int colIdx = i % colNum;
        draw.drawRect(zoneX + colIdx * subZoneW, zoneY + rowIdx * subZoneH,
                -1, subZoneW, subZoneH, true);
        if (featureVal > 0.5f) {
          draw.setColor(1, 1f, 1f, 1f);
          draw.drawRect(zoneX + colIdx * subZoneW, zoneY + rowIdx * subZoneH,
                  +1, subZoneW, subZoneH, false);
        }
      }
    }

    draw.setColor(0.8f, 0.5f, 0, 1f);
    int margin = 5;
    float newX, newY;
    float zoneVal = 0;
    for (int i = 0; i < g.getNumNodes(); i++) {
      newX = flocking[i].getX() / Config.width*(zoneW-margin*2) + zoneX + margin;
      newY = flocking[i].getY() / Config.height*(zoneH-margin*2) + zoneY + margin;

      int col = (int) Math.floor((newX - zoneX) / subZoneW);
      int row = (int) Math.floor((newY - zoneY) / subZoneH);
      if (features != null) {
        //put zone value in each boids
        zoneVal = soundValue.get(col * rowNum + row);
      }
      flocking[i].setZoneVal(zoneVal);
      draw.beginRotate(newX, newY, 0, 0, 0, (float) flocking[i].getThetaDeg());
      draw.drawLine(newX, newY, newX + margin, newY);
      draw.endRotate();
    }
    draw.drawRect(zoneX, zoneY, zoneW, zoneH);
  }

  public void computeFlocking() {
    ////sort the flocking array
    Arrays.sort(flocking, compareByX);
    for (int i = 0; i < g.getNumNodes(); i++) {
      flocking[i].move(flocking);
			if(i == 0){
				//System.out.println(flocking[i].debug);
			}
    }
  }

  void onCursorMove(Cursor cursor) {
  }

  void onCursorPress(Cursor cursor) {
  }

  void onCursorRelease(Cursor cursor) {
  }

  void onCursorDrag(Cursor cursor) {
    rotY += cursor.prevX - cursor.x;
    rotX -= cursor.prevY - cursor.y;
    rotX %= 360;
    rotY %= 360;
  }

  void onCursorClick(Cursor cursor) {
  }

  class FlockingXComparator implements Comparator<Flocking> {

    public int compare(Flocking flocking1, Flocking flocking2) {
      if (flocking1.getX() < flocking2.getX()) {
        return -1;

      } else if (flocking1.getX() > flocking2.getX()) {
        return 1;

      } else {
        return 0;
      }
    }
  }
}
