
import net.beadsproject.beads.analysis.featureextractors.FFT;
import net.beadsproject.beads.analysis.featureextractors.PowerSpectrum;
import net.beadsproject.beads.analysis.segmenters.ShortFrameSegmenter;
import net.beadsproject.beads.core.AudioContext;
import net.beadsproject.beads.core.UGen;
import net.beadsproject.beads.data.Sample;
import net.beadsproject.beads.data.SampleManager;
import net.beadsproject.beads.ugens.Gain;
import net.beadsproject.beads.ugens.SamplePlayer;

public class SoundController {

	JoglFramework fw;
	Draw draw;
	PowerSpectrum ps;

	SoundController(JoglFramework _fw, Draw _draw) {
		fw = _fw;
		draw = _draw;

		AudioContext ac = new AudioContext();

		ac.out.setGain(1f);
		UGen input = ac.getAudioInput(new int[]{0});
		ac.out.addInput(input);


		String audioFile = "audio/1234.aif";
		SampleManager.setBufferingRegime(Sample.Regime.newStreamingRegime(1000));
		SamplePlayer player = new SamplePlayer(ac, SampleManager.sample(audioFile));
		Gain g = new Gain(ac, 1);
		g.addInput(player);
		//ac.out.addInput(g);

		ShortFrameSegmenter sfs = new ShortFrameSegmenter(ac);
		sfs.addInput(ac.out);
		FFT fft = new FFT();
		ps = new PowerSpectrum();
		sfs.addListener(fft);
		fft.addListener(ps);
		ac.out.addDependent(sfs);

		ac.start();

	}

	public float[] getFeatures() {
		return ps.getFeatures();
	}

	public void draw() {
		float[] features = getFeatures();
		if (features != null) {
			int x, h;
			//int w = (int)(fw.width/features.length);
			int w = 5;
			int y = 600;
			for (int i = 0; i < 44; i++) {
				//System.out.println(i + " " + features[i] * 1000);
				x = i*w+20;
				//draw.drawRectdBottomAligned(x, y, w, features[i]);
				draw.setColor(0.8f, 0.5f, 0, 1f);
				draw.drawRectdBottomAligned(x, y, w, (float) Math.log1p(features[i])*20);
			}
		}

	}
}
