
import java.awt.Color;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.text.DecimalFormat;
import java.util.ArrayList;

public class SupershapeDisplay {

	Vector3D[][] verticies;
	double[][] vertRadius;
	double mA, n1A, n2A, n3A;
	double mB, n1B, n2B, n3B;
	double maxPhi = (float) Math.PI;
	double maxTheta = (float) Math.PI * 2;
	////shape resolution
	int phiSteps = 50;
	int thetaSteps = 100;
	int numVertices = phiSteps * thetaSteps;
	double phiStepSize = maxPhi / (phiSteps - 1);
	double thetaStepSize = maxTheta / (thetaSteps - 1);
	double phi;
	double theta;
	double[] r1 = new double[thetaSteps];
	double[] r2 = new double[phiSteps];
	double t1, t2;
	double a = 1, b = 1;
	float x, y, z;
	double r1Max, r2Max;
	double rotationTheta;
	////base rotation
	float rotX = 21;
	float rotY = 162;
	int zoom = 100;
	double counter = 0;
	ArrayList points = new ArrayList();
	SinCounter[] sinCounters;
	JoglFramework fw;
	Draw draw;
	////shape pos and size
	int scaleFactor = 15;
	//int cX;
	//int cY;
	Thread supershapeThread;
	int count = 0;
	int sendCount = 0;
	DecimalFormat Dec = new DecimalFormat("#0.00");

	SupershapeDisplay(JoglFramework _fw, Draw _draw) {
		fw = _fw;
		draw = _draw;
		mA = 2;
		n1A = 5;
		n2A = 2;
		n3A = 2;
		mB = 2;
		n1B = 5;
		n2B = 2;
		n3B = 2;
		verticies = new Vector3D[phiSteps][thetaSteps];
		vertRadius = new double[phiSteps][thetaSteps];
		sinCounters = new SinCounter[8];
		for (int i = 0; i < sinCounters.length; i++) {
			sinCounters[i] = new SinCounter(0.001d * (i + 1), 10d, 11d);
		}
		fw.stopwatchStart();
		/*
		Runnable r1 = new Runnable() {

		public void run() {
		try {
		while (true) {
		updateShape();
		Thread.sleep(10L);
		}
		} catch (InterruptedException iex) {
		}
		}
		};

		supershapeThread = new Thread(r1);
		supershapeThread.start();
		 */
		updateShape();
	}

	public void draw() {

		////auto rotate
		rotY += 0.1;
		rotX -= 0.01;
		rotX %= 360;
		rotY %= 360;

		for (int i = 0; i < sinCounters.length; i++) {
			sinCounters[i].step();
		}
		updateShape();

		draw.enableAlphaBlending(true);

		draw.setColor(0.8f, 0.5f, 0, 0.1f);
		draw.beginTranslate((float) fw.width / 2, (float) fw.height / 2, 0f);

		draw.beginRotate(0, 0, 0f, rotX, rotY, 0);
		for (int i = 0; i < phiSteps - 1; i++) {
			for (int j = 0; j < thetaSteps; j++) {
				draw.drawPoint(
								verticies[i][j].x(),
								verticies[i][j].y(),
								verticies[i][j].z(),
								10f);
//        draw.drawEllipse(
//                verticies[i][j].x(),
//                verticies[i][j].y(),
//                verticies[i][j].z(),
//                5f, 5f, 60, true);
			}
			points.clear();
		}
		draw.endRotate();
		draw.endTranslate();

		draw.enableAlphaBlending(false);
		draw.setColor(0.8f, 0.5f, 0, 1f);
		draw.beginTranslate(110f, 100f, 0);
		draw.beginScale(0.5f, 0.5f, 0.5f);
		draw.beginRotate(0f, 0f, 0f, rotX, rotY, 0);
		int inc = 6;
		for (int i = 0; i < phiSteps - inc; i += inc) {
			for (int j = -inc; j < thetaSteps; j += inc) {

				if (j == -inc) {
					points.add(verticies[i + inc][inc]);
					points.add(verticies[i + inc][0]);
				} else if (j < thetaSteps - inc) {
					points.add(verticies[i][j]);
					points.add(verticies[i][j + inc]);
					points.add(verticies[i + inc][j]);
					points.add(verticies[i + inc][j + inc]);
				} else {
					points.add(verticies[i][j]);
					points.add(verticies[i][0]);
					points.add(verticies[i + inc][j]);
					points.add(verticies[i + inc][0]);
				}
			}
			draw.drawTriStrips(points, false, false);
			points.clear();
		}
		draw.endRotate();
		draw.endScale();
		draw.endTranslate();

		draw.textRenderer2.setColor(new Color(0.8f, 0.5f, 0, 1f));
		draw.drawText3D(
						
						"r1 = pow(abs(pow(abs(cos("+Dec.format(mA)+" * theta / 4) / "+
						Dec.format(a)+"), "+Dec.format(n2A)+") pow(abs(sin("+
						Dec.format(mA)+" * theta / 4) / "+Dec.format(b)+"), "+
						Dec.format(n3A)+")), -1 / "+Dec.format(n1A)+"); " +

						"r2 = pow(abs(pow(abs(cos("+Dec.format(mB)+" * phi / 4) / "+
						Dec.format(a)+"), "+Dec.format(n2B)+") pow(abs(sin("+
						Dec.format(mB)+" * phi / 4) / "+Dec.format(b)+"), "+
						Dec.format(n3B)+")), -1 / "+Dec.format(n1B)+");", 
						2, 250, Config.height - 50, 0);

	}


	private void updateShape() {

		r1Max = 0;
		for (int j = 0; j < thetaSteps; j++) {
			theta = j * thetaStepSize - (Math.PI);
			mA = sinCounters[0].getCount();
			n1A = sinCounters[1].getCount();
			n2A = sinCounters[2].getCount();
			n3A = sinCounters[3].getCount();

			t1 = SinCosLUT.cos((float) (mA * theta / 4d)) / a;
			t1 = Math.abs(t1);
			t1 = pow(t1, n2A);
			t2 = SinCosLUT.sin((float) (mA * theta / 4d)) / b;
			t2 = Math.abs(t2);
			t2 = pow(t2, n3A);
			r1[j] = pow(Math.abs(t1 + t2), -1 / n1A);
			if (r1[j] > r1Max) {
				r1Max = r1[j];
			}
		}

		r2Max = 0;
		for (int i = 0; i < phiSteps; i++) {
			phi = i * phiStepSize - (Math.PI / 2d);
			mB = sinCounters[4].getCount();
			n1B = sinCounters[5].getCount();
			n2B = sinCounters[6].getCount();
			n3B = sinCounters[7].getCount();

			t1 = (SinCosLUT.cos((float) (mB * phi / 4d)) / a);
			t1 = Math.abs(t1);
			t1 = pow(t1, n2A);
			t2 = (SinCosLUT.sin((float) (mB * phi / 4d)) / b);
			t2 = Math.abs(t2);
			t2 = pow(t2, n3B);
			r2[i] = pow(Math.abs(t1 + t2), -1 / n1B);
			if (r2[i] > r2Max) {
				r2Max = r2[i];
			}
		}

//    if (sendCount >= 1000) {
//      String stringToSend = "m=" + m + "&n1=" + n1
//              + "&n2=" + n2 + "&n3=" + n3 + "&a=" + a + "&b=" + b + "";
//      sendData(stringToSend);
//      sendCount = 0;
//    }
//    sendCount++;

		count = 0;
		double halfPi = Math.PI / 2d;
		double r1Norm, r2Norm;
		for (int i = 0; i < phiSteps; i++) {
			phi = i * phiStepSize - (halfPi);
			r2Norm = r2[i] / (r2Max + 0.1) * scaleFactor;
			for (int j = 0; j < thetaSteps; j++) {
				theta = j * thetaStepSize - (Math.PI);
				r1Norm = r1[j] / (r1Max + 0.1) * scaleFactor;
				x = (float) (r1Norm * SinCosLUT.cos((float) theta) * r2Norm * Math.cos(phi));
				y = (float) (r1Norm * SinCosLUT.sin((float) theta) * r2Norm * Math.cos(phi));
				z = (float) (r2Norm * SinCosLUT.sin((float) phi)) * scaleFactor;
				verticies[i][j] = new Vector3D(x, y, z);
				count++;
			}
		}

	}

	public void sendData(String stringToSend) {

		try {
			URL url;
			url = new URL("http://localhost/java-js/writeData.php");

			URLConnection connection;
			connection = url.openConnection();
			connection.setDoOutput(true);

			OutputStreamWriter out = new OutputStreamWriter(connection.getOutputStream());
			out.write(stringToSend);
			out.close();

			BufferedReader in = new BufferedReader(
							new InputStreamReader(
							connection.getInputStream()));

			String decodedString;

			while ((decodedString = in.readLine()) != null) {
				System.out.println(decodedString);
			}
			in.close();

		} catch (MalformedURLException ex) {
			System.out.println(ex);
		} catch (IOException ex) {
			System.out.println(ex);
		}
	}

	void onCursorMove(Cursor cursor) {
	}

	void onCursorPress(Cursor cursor) {
	}

	void onCursorRelease(Cursor cursor) {
	}

	void onCursorDrag(Cursor cursor) {
		rotY += cursor.prevX - cursor.x;
		rotX -= cursor.prevY - cursor.y;
	}

	void onCursorClick(Cursor cursor) {
	}

	public static double pow(final double a, final double b) {
		final int x = (int) (Double.doubleToLongBits(a) >> 32);
		final int y = (int) (b * (x - 1072632447) + 1072632447);
		return Double.longBitsToDouble(((long) y) << 32);
	}
}
