
public class Vector3D {

	public float[] v = new float[3];

	public Vector3D() {
		v[0] = v[1] = v[2] = 0;
	}

	public Vector3D(float x, float y, float z) {
		v[0] = x;
		v[1] = y;
		v[2] = z;
	}

	public void copy(Vector3D V) {
		v[0] = V.v[0];
		v[1] = V.v[1];
		v[2] = V.v[2];
	}

	public float x() {
		return v[0];
	}

	public float y() {
		return v[1];
	}

	public float z() {
		return v[2];
	}

	public void setX(float newX) {
		v[0] = newX;
	}

	public void setY(float newY) {
		v[1] = newY;
	}

	public void setZ(float newZ) {
		v[2] = newZ;
	}

	public float lengthSquared() {
		return x() * x() + y() * y() + z() * z();
	}

	public float length() {
		return (float) Math.sqrt(lengthSquared());
	}

	public Vector3D negated() {
		return new Vector3D(-x(), -y(), -z());
	}

	public Vector3D normalized() {
		float l = length();
		if (l > 0) {
			float k = 1 / l; // scale factor
			return new Vector3D(k * x(), k * y(), k * z());
		} else {
			return new Vector3D(x(), y(), z());
		}
	}

	public void normalize() {
		float l = length();
		if (l != 0 && l != 1) {
			div(l);
		}
	}

	// returns the dot-product of the given vectors
	static public float dot(Vector3D a, Vector3D b) {
		return a.x() * b.x() + a.y() * b.y() + a.z() * b.z();
	}

	// returns the cross-product of the given vectors
	static public Vector3D cross(Vector3D a, Vector3D b) {
		return new Vector3D(
						a.y() * b.z() - a.z() * b.y(),
						a.z() * b.x() - a.x() * b.z(),
						a.x() * b.y() - a.y() * b.x());
	}

	// returns the sum of the given vectors
	static public Vector3D sum(Vector3D a, Vector3D b) {
		return new Vector3D(a.x() + b.x(), a.y() + b.y(), a.z() + b.z());
	}

	public void sum(Vector3D b) {
		v[0] += b.x();
		v[1] += b.y();
		v[2] += b.z();
	}

	// returns the difference of the given vectors
	static public Vector3D diff(Vector3D a, Vector3D b) {
		return new Vector3D(a.x() - b.x(), a.y() - b.y(), a.z() - b.z());
	}

	public void diff(Vector3D b) {
		v[0] -= b.x();
		v[1] -= b.y();
		v[2] -= b.z();
	}

	// returns the product of the given vector and scalar
	static public Vector3D mult(Vector3D a, float b) {
		return new Vector3D(a.x() * b, a.y() * b, a.z() * b);
	}

	// returns the product of the given vector and vector
	static public Vector3D mult(Vector3D v1, Vector3D v2, Vector3D target) {
		return new Vector3D(v1.x() * v2.x(), v1.y() * v2.y(), v1.z() * v2.z());
	}

	public void mult(float b) {
		v[0] *= b;
		v[1] *= b;
		v[2] *= b;
	}

	public void mult(Vector3D b) {
		v[0] *= b.x();
		v[1] *= b.y();
		v[2] *= b.z();
	}

	static public Vector3D div(Vector3D a, float b) {
		return new Vector3D(a.x() / b, a.y() / b, a.z() / b);
	}

	static public Vector3D div(Vector3D v1, Vector3D v2, Vector3D target) {
		return new Vector3D(v1.x() / v2.x(), v1.y() / v2.y(), v1.z() / v2.z());
	}

	public void div(Vector3D b) {
		v[0] /= b.x();
		v[1] /= b.y();
		v[2] /= b.z();
	}

	public void div(float b) {
		v[0] /= b;
		v[1] /= b;
		v[2] /= b;
	}

	static public void limit(Vector3D v1, float max) {
		if (v1.length() > max) {
			v1 = v1.normalized();
			mult(v1, max);
		}
	}

	public void limit(float max) {
		if (length() > max) {
			normalize();
			mult(max);
		}
	}

	static public float dist(Vector3D a, Vector3D b) {
		float dx = a.x() - b.x();
		float dy = a.y() - b.y();
		float dz = a.z() - b.z();
		return (float) Math.sqrt(dx * dx + dy * dy + dz * dz);
	}

	public float dist(Vector3D b) {
		float dx = x() - b.x();
		float dy = y() - b.y();
		float dz = z() - b.z();
		return (float) Math.sqrt(dx * dx + dy * dy + dz * dz);
	}

	static public Vector3D mag(Vector3D a, float b) {
		return new Vector3D(a.x() / b, a.y() / b, a.z() / b);
	}

	public Vector3D mag(float b) {
		return new Vector3D(x() / b, y() / b, z() / b);
	}

	static public float heading2D(Vector3D a) {
		float angle = (float) Math.atan2(-a.y(), a.x());
		return -1 * angle;
	}

	public float heading2D() {
		float angle = (float) Math.atan2(-y(), x());
		return -1 * angle;
	}
}
